package wechat

import (
	"encoding/json"
	"errors"
	"eta_gn/eta_report/models"
	"eta_gn/eta_report/services/alarm_msg"
	"eta_gn/eta_report/utils"
	"fmt"
	"io/ioutil"
	"net/http"
	"strings"
	"time"
)

type WxTokenResp struct {
	Ret         int
	Msg         string
	ErrMsg      string
	ErrCode     string
	Data        WxTokenData
	Success     bool `description:"true 执行成功，false 执行失败"`
	IsSendEmail bool `json:"-"`
}

type WxTokenReq struct {
	WxAppId     string `description:"公众号appId"`
	WxAppSecret string `description:"公众号appSecret"`
}

type WxTokenData struct {
	AccessToken string `description:"微信token"`
}

func GetAccessToken(tokenReq WxTokenReq) (accessToken string, err error, errMsg string) {
	if utils.EtaPubUrl == `` {
		return
	}
	url := fmt.Sprintf("%s%s", utils.EtaPubUrl, "v1/wechat/access_token")
	defer func() {
		if err != nil {
			go alarm_msg.SendAlarmMsg(errMsg, 1)
		}
	}()
	postData, err := json.Marshal(tokenReq)
	if err != nil {
		errMsg = "SendTemplateMsg json.Marshal Err:" + err.Error()
		return
	}
	body := ioutil.NopCloser(strings.NewReader(string(postData)))
	client := &http.Client{}
	req, err := http.NewRequest("POST", url, body)
	if err != nil {
		errMsg = "SendTemplateMsg http.NewRequest Err:" + err.Error()
		return
	}
	contentType := "application/json;charset=utf-8"
	req.Header.Set("Content-Type", contentType)
	req.Header.Set("Authorization", utils.EtaPubAuthorization)
	resp, err := client.Do(req)
	if err != nil {
		errMsg = "http client.Do Err:" + err.Error()
		return
	}
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		errMsg = "ioutil.ReadAll Err:" + err.Error()
		return
	}
	result := new(WxTokenResp)
	err = json.Unmarshal(b, &result)
	if err != nil {
		errMsg = "json.Unmarshal Err:" + err.Error()
		return
	}
	if result.Ret != 200 {
		err = errors.New(string(b))
		errMsg = "请求失败:" + err.Error()
		return
	}
	accessToken = result.Data.AccessToken
	if accessToken == "" {
		err = errors.New("请求accessToken失败")
		errMsg = "请求accessToken失败"
		return
	}
	return
}

func GetWxAccessToken(wxAppId, wxAppSecret string) (accessToken string, err error, errMsg string) {
	accessToken, err, errMsg = getWxAccessToken(wxAppId, wxAppSecret)
	if err != nil {
		utils.FileLog.Info(fmt.Sprintf("获取Token失败,err：%s,errMsg：%s", err.Error(), errMsg))
		return
	}
	return
}

func getWxAccessToken(wxAppId, wxAppSecret string) (accessToken string, err error, errMsg string) {
	redisKey := getRedisKeyByAppid(wxAppId)
	if redisKey == `` {
		errMsg = "未配置缓存key"
		err = errors.New(errMsg)
		return
	}

	accessToken, err = utils.Rc.RedisString(redisKey)
	if accessToken != "" {
		return
	}

	return refreshWxAccessToken(wxAppId, wxAppSecret)
}

func refreshWxAccessToken(wxAppId, wxAppSecret string) (accessToken string, err error, errMsg string) {
	fmt.Println("强制刷新" + wxAppId + "微信token")
	defer func() {
		if errMsg != `` {
			utils.FileLog.Info(fmt.Sprintf("强制刷新%s微信token异常：%s", wxAppId, errMsg))
		}
	}()

	if wxAppId == "" || wxAppSecret == "" {
		errMsg = fmt.Sprintf("必要信息为空, wxAppId: %s, wxAppSecret: %s", wxAppId, wxAppSecret)
		err = errors.New(errMsg)
		return
	}

	redisKey := getRedisKeyByAppid(wxAppId)
	if redisKey == `` {
		errMsg = "未配置缓存key"
		err = errors.New(errMsg)
		return
	}

	wxAccessToken, tmpErr := models.GetWxToken(wxAppId, wxAppSecret)
	if tmpErr != nil {
		err = tmpErr
		errMsg = "通过微信接口获取accessToken失败 Err:" + err.Error()
		return
	}

	if wxAccessToken.AccessToken == "" {
		errMsg = "微信返回的accessToken异常： Err:" + wxAccessToken.Errmsg
		err = errors.New(errMsg)
		return
	}

	accessToken = wxAccessToken.AccessToken

	redisTimeExpire := time.Duration(wxAccessToken.ExpiresIn-600) * time.Second
	err = utils.Rc.Put(redisKey, accessToken, redisTimeExpire)
	if err != nil {
		errMsg = "更新redis中的accessToken失败 Err:" + err.Error()
		return
	}

	return
}

func getRedisKeyByAppid(wxAppId string) (redisKey string) {
	redisKey = utils.ETA_WX_ACCESS_TOEKN + wxAppId
	return redisKey
}
